@@ -38,20 +38,34 @@ const int MEDIUM_FONT_METRICS[256] = { 8, 8, 8, 8, 8, 8, 8, 8, 8, 56, 0, 8, 8, 0
 
 GLfloat boxVertices[12] = { 0.0f, 0.0f, 0.0f, 0.0f, 256.0f, 0.0f, 256.0f, 256.0f, 0.0f, 256.0f, 0.0f, 0.0f, };
 
+GLfloat fakeFogColorsLandscape[16] = { 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.01f, 0.0f, 0.0f, 0.0f, 0.01f };
+GLfloat fakeFogColorsPortrait[16] = { 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.01f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.01f };
+
+int debugTilesGridInitialized = FALSE;
+GLfloat debugTilesGrid[(TILES_X + TILES_Y + 2) * 3 * 2];
+
 GLfloat quadVertices[12] = { 0.0f, 0.0f, 0.0f, /* Bottom Left Of The Quad */
 1.0f, 0.0f, 0.0f, /* Bottom Right Of The Quad */
 0.0f, 1.0f, 0.0f, /* Top Left Of The Quad */
 1.0f, 1.0f, 0.0f, /* Top Right Of The Quad */
 };
 
-const GLfloat texCoords[][8] = { { 0.0f, 0.0f, /* Bottom Left Of The Texture */
+GLfloat texCoordsLandscape[][8] = { { 0.0f, 0.0f, /* Bottom Left Of The Texture */
 1.0f, 0.0f, /* Bottom Right Of The Texture */
 0.0f, 1.0f, /* Top Left Of The Texture */
 1.0f, 1.0f, /* Top Right Of The Texture */
 } };
 
-SDL_Surface* loadTextureFromFile(char *filename);
-GLuint createTexture(GLushort * pixels4444, int w, int h);
+GLfloat texCoordsPortrait[][8] = { { 1.0f, 0.0f, /* Top Right Of The Texture */
+1.0f, 1.0f, /* Bottom Right Of The Texture */
+0.0f, 0.0f, /* Bottom Left Of The Texture */
+0.0f, 1.0f, /* Top Left Of The Texture */
+
+} };
+
+SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile);
+
+GLuint createTexture(GLushort * pixels4444, int w, int h, int isTile);
 
 char buf[500];
 
@@ -64,7 +78,7 @@ int stringWidth(char * str) {
     return result;
 }
 
-void drawString(char* str, GLfloat r, GLfloat g, GLfloat b) {
+void drawString(char* str, GLfloat r, GLfloat g, GLfloat b, int useMask) {
     int i, len, charTileSize, xpos = 0;
     //    int j;
     len = strlen(str);
@@ -140,13 +154,15 @@ void drawString(char* str, GLfloat r, GLfloat g, GLfloat b) {
         //            fprintf(stderr, "textureCoords[%d] = %g\n", i * 12 + j, fontTextureCoords[i * 12 + j]);
         //        }
     }
-    glEnable(GL_BLEND);
-    glBlendFunc(GL_DST_COLOR, GL_ZERO);
-    glColor4f(1.0, 1.0, 1.0, 1.0);
-    glBindTexture(GL_TEXTURE_2D, fontMediumMask.name);
-    glVertexPointer(3, GL_FLOAT, 0, vertices);
-    glTexCoordPointer(2, GL_FLOAT, 0, fontTextureCoords);
-    glDrawArrays(GL_TRIANGLES, 0, len * 6);
+    if (useMask) {
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_DST_COLOR, GL_ZERO);
+        glColor4f(1.0, 1.0, 1.0, 1.0);
+        glBindTexture(GL_TEXTURE_2D, fontMediumMask.name);
+        glVertexPointer(3, GL_FLOAT, 0, vertices);
+        glTexCoordPointer(2, GL_FLOAT, 0, fontTextureCoords);
+        glDrawArrays(GL_TRIANGLES, 0, len * 6);
+    }
 
     glColor4f(r, g, b, 1.0);
     glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
@@ -163,7 +179,65 @@ void drawString(char* str, GLfloat r, GLfloat g, GLfloat b) {
 
 float color = 1.0;
 
-void drawTile(t_tile *tile) {
+void drawBoundingBox() {
+    glDisable(GL_BLEND);
+    glBindTexture(GL_TEXTURE_2D, 0);
+    glColor4f(0, 0, 1, 1.0f);
+
+    GLfloat boundingTrapezoidVertices[12] = {
+    // first point:
+            (canvas.boundingTrapezoid[0].tilex - canvas.tilex) * TILE_SIZE + canvas.boundingTrapezoid[0].x, (canvas.boundingTrapezoid[0].tiley
+                    - canvas.tiley) * TILE_SIZE + canvas.boundingTrapezoid[0].y, 0,
+            // second point:
+            (canvas.boundingTrapezoid[1].tilex - canvas.tilex) * TILE_SIZE + canvas.boundingTrapezoid[1].x, (canvas.boundingTrapezoid[1].tiley
+                    - canvas.tiley) * TILE_SIZE + canvas.boundingTrapezoid[1].y, 0,
+            // third point:
+            (canvas.boundingTrapezoid[2].tilex - canvas.tilex) * TILE_SIZE + canvas.boundingTrapezoid[2].x, (canvas.boundingTrapezoid[2].tiley
+                    - canvas.tiley) * TILE_SIZE + canvas.boundingTrapezoid[2].y, 0,
+            // fourth point:
+            (canvas.boundingTrapezoid[3].tilex - canvas.tilex) * TILE_SIZE + canvas.boundingTrapezoid[3].x, (canvas.boundingTrapezoid[3].tiley
+                    - canvas.tiley) * TILE_SIZE + canvas.boundingTrapezoid[3].y, 0, };
+
+    glVertexPointer(3, GL_FLOAT, 0, boundingTrapezoidVertices);
+    glDrawArrays(GL_LINE_LOOP, 0, 4);
+
+}
+
+void drawTile_debug(t_tile *tile) {
+    switch (tile -> state) {
+        case STATE_EMPTY:
+            glColor4f(0,0,0, 0.5);
+            break;
+        case STATE_ERROR:
+            glColor4f(1,0,0, 0.5);
+            break;
+        case STATE_LOADING:
+            glColor4f(0,0,1, 0.5);
+            break;
+        case STATE_LOADED:
+            glColor4f(0,1,0, 0.5);
+            break;
+        case STATE_GL_TEXTURE_NOT_CREATED:
+            glColor4f(0,0.2,0, 0.5);
+            break;
+        case STATE_SCALED_LOADING:
+            glColor4f(0.5,0.5,0.5, 0.5);
+            break;
+        case STATE_LOADED_SYNC_WAIT:
+            glColor4f(1,0,1, 0.5);
+            break;
+
+    }
+    glVertexPointer(3, GL_FLOAT, 0, quadVertices);
+    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+
+    //        sprintf(buf, "%d", tile -> state);
+    //        glScalef(10.0, 10.0, 10.0);
+    //        drawString(buf, 0.1f, .0, 1.0, TRUE);
+    //        glScalef(.1, .1, .1);
+}
+
+void drawTile(t_tile* tile) {
     int i;
 
     switch (tile->state) {
@@ -173,15 +247,22 @@ void drawTile(t_tile *tile) {
             //            glDrawArrays(GL_LINE_LOOP, 0, 4);
             break;
         case STATE_ERROR:
-            //            glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
-            //            glVertexPointer(3, GL_FLOAT, 0, boxVertices[0]);
-            //            glDrawArrays(GL_LINE_LOOP, 0, 4);
+            glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
+            setBoxSize(TILE_SIZE, TILE_SIZE, boxVertices);
+            glVertexPointer(3, GL_FLOAT, 0, boxVertices);
+            glDrawArrays(GL_LINE_LOOP, 0, 4);
+            if (tile -> errorMessage) {
+                sprintf(buf, "Error: %s", tile->errorMessage);
+                drawString(buf, 1.0f, .2, .2, TRUE);
+            }
+
             break;
         case STATE_LOADING:
             //            glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
             //            glVertexPointer(3, GL_FLOAT, 0, boxVertices[0]);
             //            glDrawArrays(GL_LINE_LOOP, 0, 4);
             break;
+
         case STATE_LOADED:
             //            tile -> transitionTime = 1500;
             if (nowMillis - tile->stateChangeTime < tile->transitionTime) {
@@ -189,40 +270,53 @@ void drawTile(t_tile *tile) {
                 //color = 1.0 - sqrt(sqrt(1.0 - color));
             } else {
                 color = 1.0;
-
-                if (tile->oldTexture[0] > 0) {
-                    if (tile->oldTexture[1] == 0) {
-                        int j, count = 0;
-                        for (i = 0; i < TILES_X; i++) {
-                            for (j = 0; j < TILES_Y; j++) {
-                                if (tiles[i][j][currentTilesIdx] -> lat != tile -> lat && tiles[i][j][currentTilesIdx] -> lon != tile -> lon
-                                        && tiles[i][j][currentTilesIdx] -> oldTexture[0] == tile -> oldTexture[0]) {
-                                    count++;
+                int j, k, count;
+                TDeallocationData *data = NULL;
+                for (i = 0; i < 4; i++) {
+                    if (tile->oldTexture[i]) {
+                        count = 0;
+                        if (i == 0) {
+                            for (j = 0; j < TILES_X; j++) {
+                                for (k = 0; k < TILES_Y; k++) {
+                                    if (tiles[j][k][currentTilesIdx] -> visible && tiles[j][k][currentTilesIdx] -> oldTexture[0]
+                                            == tile -> oldTexture[0]) {
+                                        count++;
+                                    }
                                 }
                             }
                         }
-                        if (count == 0) {
-                            glDeleteTextures(1, &tile->oldTexture[0]);
-                            //                            free(tile -> oldPixels4444[0]);
-                            //                            tile -> oldPixels4444[0] = NULL;
-                            //                            SDL_FreeSurface(tile -> oldTmpSurface[0]);
-                            //                            tile -> oldTmpSurface[0] = NULL;
-                        }
-                        tile->oldTexture[0] = 0;
-                    } else {
-                        for (i = 0; i < 4; i++) {
-                            if (tile->oldTexture[i]) {
-                                glDeleteTextures(1, &tile->oldTexture[i]);
-                                //                                tile->oldTexture[i] = 0;
-                                //                                free(tile -> oldPixels4444[i]);
-                                //                                tile -> oldPixels4444[i] = NULL;
-                                //                                SDL_FreeSurface(tile -> oldTmpSurface[i]);
-                                //                                tile -> oldTmpSurface[i] = NULL;
+//                        fprintf(stderr, "(%d, %d) texture = %d, oldpixels[%d] = %p, count = %d\n", tile->tilex, tile->tiley, tile -> oldTexture[i],
+//                                i, tile -> oldPixels4444[i], count);
+                        GLushort * oldpixels4444 = tile -> oldPixels4444[i];
+                        if (count < 2) {
+                            if (i == 0) {
+                                for (j = 0; j < TILES_X; j++) {
+                                    for (k = 0; k < TILES_Y; k++) {
+                                        if (tiles[j][k][currentTilesIdx] -> oldPixels4444[0] == oldpixels4444) {
+                                            tiles[j][k][currentTilesIdx] -> oldPixels4444[0] = NULL;
+                                        }
+                                    }
+                                }
                             }
+
+                            glDeleteTextures(1, &tile->oldTexture[i]);
+                            if (data == NULL) {
+                                data = calloc(1, sizeof(TDeallocationData));
+                                data -> frame = frame;
+                            }
+                            data -> oldPixels4444[i] = oldpixels4444;
+                        }
+                        tile->oldTexture[i] = 0;
+
+                        if (tile -> oldPixels4444[i] != NULL) {
+//                            fprintf(stderr, "############################### 1\n");
+                            tile -> oldPixels4444[i] = NULL;
                         }
                     }
                 }
-
+                if (data != NULL) {
+                    g_queue_push_tail(deallocationQueue, data);
+                }
             }
 
             if (color < 1.0) {
@@ -247,10 +341,10 @@ void drawTile(t_tile *tile) {
             glBindTexture(GL_TEXTURE_2D, tile->texture);
             setQuadSize(TILE_SIZE, TILE_SIZE, quadVertices);
             glVertexPointer(3, GL_FLOAT, 0, quadVertices);
-            glTexCoordPointer(2, GL_FLOAT, 0, texCoords[0]);
+            glTexCoordPointer(2, GL_FLOAT, 0, texCoordsLandscape[0]);
             glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
             break;
-        case STATE_TEXTURE_BIND_WAIT:
+
         case STATE_LOADED_SYNC_WAIT:
         case STATE_SCALED_LOADING:
             glDisable(GL_BLEND);
@@ -267,17 +361,20 @@ void drawTile(t_tile *tile) {
                 }
             }
             break;
+        case STATE_GL_TEXTURE_NOT_CREATED:
+            //            fprintf(stderr, "internal error: trying to draw tile in state STATE_GL_TEXTURE_NOT_CREATED\n");
+            break;
     }
-    if (canvas.showGrid) {
+    if (options.showGrid) {
         glDisable(GL_BLEND);
         setBoxSize(TILE_SIZE, TILE_SIZE, boxVertices);
         glBindTexture(GL_TEXTURE_2D, 0);
-        glColor4f(0.7f, .7f, 0.7f, .9f);
+        glColor4f(0.7f, .7f, 0.7f, 1.0f);
         glVertexPointer(3, GL_FLOAT, 0, boxVertices);
         glDrawArrays(GL_LINE_LOOP, 0, 4);
 
-        sprintf(buf, "%d, %d", tile->lat, tile -> lon);
-        drawString(buf, 0.1f, .0, 1.0);
+        sprintf(buf, "%d, %d", tile->tilex, tile -> tiley);
+        drawString(buf, 0.1f, .0, 1.0, TRUE);
     }
 }
 
@@ -288,34 +385,32 @@ int initGL(GLvoid) {
     context = SDL_GLES_CreateContext();
     SDL_GLES_MakeCurrent(context);
 
-    /* Enable Texture Mapping ( NEW ) */
     glEnable(GL_TEXTURE_2D);
 
-    /* Set the background black */
-    glClearColor(.0f, .0f, .0f, 1.0f);
+    glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
     glEnable(GL_CLEAR);
 
     glEnableClientState(GL_VERTEX_ARRAY);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
+    glMatrixMode(GL_PROJECTION);
     glDisable(GL_DEPTH_TEST);
     glDisable(GL_CULL_FACE);
     glViewport(0, 0, (GLint) SCREEN_WIDTH, (GLint) SCREEN_HEIGHT);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    //glOrthof(0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -10000.0, 10000.0);
     glFrustumf(-200, 200, 120, -120, 50.0, 600);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
 
-    GLfloat fogColor[4] = { 0.1f, 0.1f, 0.1f, 1.0f };
-    glFogx(GL_FOG_MODE, GL_LINEAR); // Fog Mode
-    glFogfv(GL_FOG_COLOR, fogColor); // Set Fog Color
-    glFogf(GL_FOG_DENSITY, .1f); // How Dense Will The Fog Be
-    glHint(GL_FOG_HINT, GL_DONT_CARE); // Fog Hint Value
-    glFogf(GL_FOG_START, 100.0f); // Fog Start Depth
-    glFogf(GL_FOG_END, 200.0f); // Fog End Depth
-    glEnable(GL_FOG);
+    //    GLfloat fogColor[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
+    //    glFogx(GL_FOG_MODE, GL_LINEAR); // Fog Mode
+    //    glFogfv(GL_FOG_COLOR, fogColor); // Set Fog Color
+    //    glFogf(GL_FOG_DENSITY, .1f); // How Dense Will The Fog Be
+    //    glHint(GL_FOG_HINT, GL_FASTEST); // Fog Hint Value
+    //    glFogf(GL_FOG_START, 100.0f); // Fog Start Depth
+    //    glFogf(GL_FOG_END, 400.0f); // Fog End Depth
+    //    glEnable(GL_FOG);
     //    const GLubyte *extensions = glGetString(GL_EXTENSIONS);
     //    fprintf(stderr, "GL extensions: %s\n", (const char*) extensions);
 
@@ -323,61 +418,171 @@ int initGL(GLvoid) {
 }
 
 GLfloat seconds;
-GLfloat fps;
 
-void drawGLScene(GLvoid) {
-    static GLint oldTicks = 0;
-    static GLint frames = 0;
-    int i, j, k;
+void setupGlScene() {
+    int pixelPerfect;
 
     glClear(GL_COLOR_BUFFER_BIT);
 
     glLoadIdentity();
-    glTranslatef(-(canvas.scale - 1.0) * SCREEN_WIDTH / 2 - SCREEN_WIDTH / 2, -(canvas.scale - 1.0) * SCREEN_HEIGHT / 2 - SCREEN_HEIGHT / 2, -100.0f);
-    glScalef(canvas.scale, canvas.scale, canvas.scale);
 
-    if ( fabs(canvas.rotx) > 0.02 || canvas.rotz > 0.02) {
-        glTranslatef(SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2, 0);
+    GLfloat scale = canvas.scale;
+    if (fabs(canvas.rotx) > 0.1) {
+        scale += fabs(canvas.rotx) / 35.0;
+    }
+    if (fabs(canvas.roty) > 0.1) {
+        scale += fabs(canvas.roty) / 25.0;
+    }
+    glTranslatef(-(scale - 1.0) * SCREEN_WIDTH / 2 - SCREEN_WIDTH / 2, -(scale - 1.0) * SCREEN_HEIGHT / 2 - SCREEN_HEIGHT / 2, -100.0f);
+    glScalef(scale, scale, scale);
+
+    glTranslatef(SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2, 0);
+    if (fabs(canvas.rotx) < 0.01 && (fabs(canvas.rotz) < 0.01 || fabs(canvas.rotz + 90.0) < 0.01) && fabs(canvas.roty) < 0.01) {
+        glRotatef(canvas.rotz, 0.0, 0.0, 1.0);
+        pixelPerfect = TRUE;
+    } else {
         glRotatef(canvas.rotx, 1.0, 0.0, 0.0);
-        glRotatef(canvas.rotz, 0, 0.0, 1.0);
-        glTranslatef(-SCREEN_WIDTH / 2, -SCREEN_HEIGHT / 2, 0);
-        k = TRUE;
-    } else {
-        k = FALSE;
+        glRotatef(canvas.roty, 0.0, 1.0, 0.0);
+        glRotatef(canvas.rotz, 0.0, 0.0, 1.0);
+        pixelPerfect = FALSE;
     }
+    glTranslatef(-SCREEN_WIDTH / 2, -SCREEN_HEIGHT / 2, 0);
 
-    if (fabs(canvas.rotx) > 5) {
-        glEnable(GL_FOG);
-    } else {
-        glDisable(GL_FOG);
-    }
-
-    if (canvas.scale - 1.0 > 0.01 || k) {
-        glTranslatef(canvas.x - TILE_SIZE, canvas.y - TILE_SIZE, 0.0f);
-    } else {
+    if (canvas.scale - 1.0 < 0.01 && pixelPerfect) {
+        // for pixel perfect representation of tiles in 2d mode
         glTranslatef((int) canvas.x - TILE_SIZE, (int) canvas.y - TILE_SIZE, 0.0f);
+    } else {
+        glTranslatef(canvas.x - TILE_SIZE, canvas.y - TILE_SIZE, 0.0f);
     }
 
+    glGetFloatv(GL_PROJECTION_MATRIX, projectionMatrix);
+    glGetFloatv(GL_MODELVIEW_MATRIX, modelMatrix);
+    glGetIntegerv(GL_VIEWPORT, viewPort);
+}
+
+void drawGLScene() {
+    static GLint oldTicks = 0;
+    static GLint frames = 0;
+    int i, j;
+
     for (i = 0; i < TILES_X; i++) {
         for (j = 0; j < TILES_Y; j++) {
-            glTranslatef(i * TILE_SIZE, j * TILE_SIZE, 0);
-            drawTile(tiles[i][j][currentTilesIdx]);
-            glTranslatef(-i * TILE_SIZE, -j * TILE_SIZE, 0);
+            if (tiles[i][j][currentTilesIdx] -> visible == TRUE) {
+                glTranslatef((i - getCanvasShiftX()) * TILE_SIZE, (j - getCanvasShiftY()) * TILE_SIZE, 0);
+                drawTile(tiles[i][j][currentTilesIdx]);
+                glTranslatef(-(i - getCanvasShiftX()) * TILE_SIZE, -(j - getCanvasShiftY()) * TILE_SIZE, 0);
+            }
         }
     }
+
     if (position -> status != UI_HIDDEN) {
         drawUiElement(position);
     }
 
+    if (options.showDebugTiles) {
+        glLoadIdentity();
+        glTranslatef(300, 0, -100);
+        glScalef(0.05, 0.05, 0.05);
+
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+        setQuadSize(TILE_SIZE, TILE_SIZE, quadVertices);
+        glBindTexture(GL_TEXTURE_2D, 0);
+
+        glTranslatef(-getCanvasShiftX() * TILE_SIZE, -getCanvasShiftY() * TILE_SIZE, 0);
+
+        for (i = 0; i < TILES_X; i++) {
+            for (j = 0; j < TILES_Y; j++) {
+                glTranslatef(i * TILE_SIZE, j * TILE_SIZE, 0);
+                drawTile_debug(tiles[i][j][currentTilesIdx]);
+                glTranslatef(-i * TILE_SIZE, -j * TILE_SIZE, 0);
+            }
+        }
+
+        if (!debugTilesGridInitialized) {
+            for (i = 0; i < TILES_X + 1; i++) {
+                debugTilesGrid[i * 6] = i * TILE_SIZE;
+                debugTilesGrid[i * 6 + 1] = 0.0;
+                debugTilesGrid[i * 6 + 2] = 0.0;
+
+                debugTilesGrid[i * 6 + 3] = i * TILE_SIZE;
+                debugTilesGrid[i * 6 + 4] = TILES_Y * TILE_SIZE;
+                debugTilesGrid[i * 6 + 5] = 0.0;
+            }
+
+            for (i = 0; i < TILES_Y + 1; i++) {
+                debugTilesGrid[(i + TILES_X + 1) * 6] = 0.0;
+                debugTilesGrid[(i + TILES_X + 1) * 6 + 1] = i * TILE_SIZE;
+                debugTilesGrid[(i + TILES_X + 1) * 6 + 2] = 0.0;
+
+                debugTilesGrid[(i + TILES_X + 1) * 6 + 3] = TILES_X * TILE_SIZE;
+                debugTilesGrid[(i + TILES_X + 1) * 6 + 4] = i * TILE_SIZE;
+                debugTilesGrid[(i + TILES_X + 1) * 6 + 5] = 0.0;
+            }
+            debugTilesGridInitialized = TRUE;
+        }
+        glColor4f(0.4, 0.4, 0.4, .7);
+        glVertexPointer(3, GL_FLOAT, 0, debugTilesGrid);
+        glDrawArrays(GL_LINES, 0, (TILES_X + TILES_Y + 2) * 2);
+
+        drawBoundingBox();
+        glScalef(20, 20, 20);
+//        struct mallinfo meminfo = mallinfo();
+//        glTranslatef(-30, 175, 0);
+//        sprintf(buf, "mem: %.3f MB", meminfo.uordblks / 1024.0 / 1024.0);
+//        drawString(buf, 1.0, 0, 0, TRUE);
+
+    }
+
     glLoadIdentity();
     glTranslatef(-SCREEN_WIDTH / 2, -SCREEN_HEIGHT / 2, -100);
-    drawUI();
+
+    glEnable(GL_BLEND);
+    glBindTexture(GL_TEXTURE_2D, 0);
+    glBlendFunc(GL_SRC_COLOR, GL_ZERO);
+    //     landscape fake fog
+    if (fabs(canvas.rotx) > 0.01) {
+        glTranslatef(0, -((LANDSCAPE_ROTATION_X - canvas.rotx) / LANDSCAPE_ROTATION_X) * 200, 0);
+        glEnableClientState(GL_COLOR_ARRAY);
+        glColorPointer(4, GL_FLOAT, 0, fakeFogColorsLandscape);
+        setQuadSize(SCREEN_WIDTH, 200, quadVertices);
+        glVertexPointer(3, GL_FLOAT, 0, quadVertices);
+        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+        glDisableClientState(GL_COLOR_ARRAY);
+        glTranslatef(0, +((LANDSCAPE_ROTATION_X - canvas.rotx) / LANDSCAPE_ROTATION_X) * 200, 0);
+    }
+
+    // portrait fake fog
+    if (fabs(canvas.roty) > 0.01) {
+        GLfloat fogStart = ((PORTRAIT_ROTATION_Y + canvas.roty) / PORTRAIT_ROTATION_Y) * 250 - 50;
+        if (fogStart < 0.0) {
+            glColor4f(0.0, 0.0, 0.0, 1.0);
+            setQuadSize(-fogStart, SCREEN_HEIGHT, quadVertices);
+            glVertexPointer(3, GL_FLOAT, 0, quadVertices);
+            glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+        }
+
+        glTranslatef(-fogStart, 0, 0);
+        glEnableClientState(GL_COLOR_ARRAY);
+        glColorPointer(4, GL_FLOAT, 0, fakeFogColorsPortrait);
+        setQuadSize(200, SCREEN_HEIGHT, quadVertices);
+        glVertexPointer(3, GL_FLOAT, 0, quadVertices);
+        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+        glDisableClientState(GL_COLOR_ARRAY);
+        glTranslatef(fogStart, 0, 0);
+    }
+
+    drawUiElems();
+    drawStatusBar(LANDSCAPE);
+    glLoadIdentity();
+    glTranslatef(-SCREEN_WIDTH / 2, -SCREEN_HEIGHT / 2, -100);
+    drawStatusBar(PORTRAIT);
 
     glLoadIdentity();
     glTranslatef(15, 15, -100);
-    if (canvas.showCoordinates) {
+    if (options.showCoordinates) {
         sprintf(buf, "%02.6f %02.6f", canvas.center.latitude, canvas.center.longitude);
-        drawString(buf, 0.4, 0.4, 0.4);
+        drawString(buf, 0.4, 0.4, 0.4, TRUE);
     }
 
     frames++;
@@ -388,11 +593,8 @@ void drawGLScene(GLvoid) {
             fps = frames / seconds;
             oldTicks = t;
             frames = 0;
-            //fprintf(stderr, "fps: %f rotx: %f\n", fps, rotx);
+            //            fprintf(stderr, "fps: %f\n", fps);
         }
-        sprintf(buf, "FPS: %.1f", fps);
-        glTranslatef(SCREEN_WIDTH / 2 - 85, -SCREEN_HEIGHT / 2 - 10, 0);
-        drawString(buf, 1.0, 1.0, 1.0);
     }
 
     SDL_GLES_SwapBuffers();
