@@ -57,18 +57,43 @@ void processMouse() {
 void processKeyDown(SDL_KeyboardEvent key) {
     switch (key.keysym.sym) {
         case SDLK_g:
-            canvas.showGrid = 1 - canvas.showGrid;
+            options.showGrid = 1 - options.showGrid;
+            break;
+        case SDLK_h:
+            options.showDebugTiles = 1 - options.showDebugTiles;
             break;
         case SDLK_c:
-            canvas.showCoordinates = 1 - canvas.showCoordinates;
+            options.showCoordinates = 1 - options.showCoordinates;
             break;
         case SDLK_SPACE:
-            followingMypos = 1 - followingMypos;
+            canvas.followingMypos = 1 - canvas.followingMypos;
             break;
         case SDLK_x:
-            canvas.accelerometerEnabled = 1 - canvas.accelerometerEnabled;
+            options.accelerometerEnabled = 1 - options.accelerometerEnabled;
             accelerometer.performCalibration = TRUE;
             break;
+        case SDLK_m:
+            options.mipmapToggle = 1 - options.mipmapToggle;
+            recreateTiles();
+            break;
+        case SDLK_n: {
+            GList *provider = tileProviders;
+            while (provider != NULL) {
+                TileProvider *tileProvider = provider -> data;
+                if (canvas.provider == tileProvider) {
+                    if (provider -> next != NULL) {
+                        canvas.provider = provider -> next -> data;
+                    } else {
+                        canvas.provider = tileProviders -> data;
+                    }
+//                    fprintf(stderr, "new provider: %s\n", canvas.provider -> name);
+                    break;
+                }
+                provider = provider -> next;
+            }
+            recreateTiles();
+        }
+            break;
         default:
             //            fprintf(stderr, "unhandled key down: key name %s\n", SDL_GetKeyName(key.keysym.sym));
             break;
@@ -89,29 +114,29 @@ void processKeyboard() {
         if (canvas.viewMode == VIEW_3D) {
             canvas.drotz += .2;
         } else {
-            followingMypos = 0;
-            canvas.dx += 0.2 * sin((canvas.rotz + 90) * M_PI / 180.0);
-            canvas.dy += 0.2 * cos((canvas.rotz + 90) * M_PI / 180.0);
+            canvas.followingMypos = 0;
+            canvas.dx += 0.2 * sin((canvas.rotz + 90 + 90 * options.orientation) * M_PI / 180.0);
+            canvas.dy += 0.2 * cos((canvas.rotz + 90 + 90 * options.orientation) * M_PI / 180.0);
         }
     }
     if (keys[SDLK_RIGHT]) {
         if (canvas.viewMode == VIEW_3D) {
             canvas.drotz -= .2;
         } else {
-            followingMypos = 0;
-            canvas.dx += 0.2 * sin((canvas.rotz - 90) * M_PI / 180.0);
-            canvas.dy += 0.2 * cos((canvas.rotz - 90) * M_PI / 180.0);
+            canvas.followingMypos = 0;
+            canvas.dx += 0.2 * sin((canvas.rotz - 90 + 90 * options.orientation) * M_PI / 180.0);
+            canvas.dy += 0.2 * cos((canvas.rotz - 90 + 90 * options.orientation) * M_PI / 180.0);
         }
     }
     if (keys[SDLK_UP]) {
-        followingMypos = 0;
-        canvas.dx += 0.2 * sin(canvas.rotz * M_PI / 180.0);
-        canvas.dy += 0.2 * cos(canvas.rotz * M_PI / 180.0);
+        canvas.followingMypos = 0;
+        canvas.dx += 0.2 * sin((canvas.rotz + 90 * options.orientation) * M_PI / 180.0);
+        canvas.dy += 0.2 * cos((canvas.rotz + 90 * options.orientation) * M_PI / 180.0);
     }
     if (keys[SDLK_DOWN]) {
-        followingMypos = 0;
-        canvas.dx += 0.2 * sin((canvas.rotz + 180) * M_PI / 180.0);
-        canvas.dy += 0.2 * cos((canvas.rotz + 180) * M_PI / 180.0);
+        canvas.followingMypos = 0;
+        canvas.dx += 0.2 * sin((canvas.rotz + 180 + 90 * options.orientation) * M_PI / 180.0);
+        canvas.dy += 0.2 * cos((canvas.rotz + 180 + 90 * options.orientation) * M_PI / 180.0);
     }
     if (keys[SDLK_KP_ENTER]) {
         tileEngineZoomIn();
@@ -120,27 +145,27 @@ void processKeyboard() {
         tileEngineZoomOut();
     }
 
-    if (keys[SDLK_a]) {
-        if (canvas.viewMode == VIEW_3D) {
-            if (canvas.destRotx < 90) {
-                canvas.destRotx += .1;
-            }
-        }
-    }
-    if (keys[SDLK_z]) {
-        if (canvas.viewMode == VIEW_3D) {
-            if (canvas.destRotx > 0) {
-                canvas.destRotx -= .1;
-            }
-        }
-    }
+    //    if (keys[SDLK_a]) {
+    //        if (canvas.viewMode == VIEW_3D) {
+    //            if (canvas.destRotx < 50) {
+    //                canvas.destRotx += .1;
+    //            }
+    //        }
+    //    }
+    //    if (keys[SDLK_z]) {
+    //        if (canvas.viewMode == VIEW_3D) {
+    //            if (canvas.destRotx > 0) {
+    //                canvas.destRotx -= .1;
+    //            }
+    //        }
+    //    }
 }
 
 int liqaccel_read(int *ax, int *ay, int *az);
 
 void processAccelerometer() {
-    if (canvas.accelerometerEnabled == 0) {
-        return;
+    if (accelerometer.performCalibration) {
+        ocnt = 0;
     }
     liqaccel_read(&accelerometer.x, &accelerometer.y, &accelerometer.z);
     if (accelerometer.performCalibration) {
@@ -152,6 +177,5 @@ void processAccelerometer() {
     accelerometer.x -= accelerometer.calibrateX;
     accelerometer.y -= accelerometer.calibrateY;
     accelerometer.z -= accelerometer.calibrateZ;
-    //fprintf(stderr, "accelerometer: %d, %d, %d\n", accelerometer.x, accelerometer.y, accelerometer.z);
     tileEngineProcessAccelerometer();
 }
