@@ -16,6 +16,7 @@
 #include <strings.h>
 #include <memory.h>
 #include <curl/curl.h>
+#include <malloc.h>
 
 #include <SDL/SDL.h>
 #include <SDL/SDL_image.h>
@@ -32,39 +33,19 @@
 #define SCREEN_BPP     16
 #define TILE_SIZE     256
 
-//#define TRUE  1
-//#define FALSE 0
+#define PORTRAIT_ROTATION_Y 14
+#define LANDSCAPE_ROTATION_X 20
+
+// how much sattelite bars are drawn in status bar
+#define MAX_SATTELITES 12
 
 // how many tiles horizontally
-#define TILES_X (SCREEN_WIDTH/TILE_SIZE + 2)
+#define TILES_X 10
 
 // how many tiles vertically
-#define TILES_Y (SCREEN_HEIGHT/TILE_SIZE + 2)
+#define TILES_Y 10
 
-//#define DEBUG 0
-//#define LOG_INFO 1
-//#define LOG_ERROR 2
-//
-//#define LEVEL 0
-//
-//SDL_mutex * logMutex = NULL;
-
-//void logmsg(int level, const char* msg, ...) {
-////    if (logMutex == NULL) {
-////        logMutex = SDL_CreateMutex();
-////    }
-//    __attribute__ ((msg (printf, 2, 3)));
-//    if (level >= LEVEL) {
-//        char buf[500];
-//        va_list ap;
-//        va_start(ap, msg);
-//        vsprintf(buf, msg, ap);
-//        va_end(ap);
-////        SDL_mutexP(logMutex);
-//        fprintf(stderr, buf);
-////        SDL_mutexV(logMutex);
-//    }
-//}
+int tileCoordinateVisibility[TILES_X+1][TILES_Y+1];
 
 int quit = FALSE;
 long nowMillis = 0, previousMillis = 0, diffMillis = 0;
@@ -75,9 +56,29 @@ long lastBatteryRefresh;
 int syncLoadedTilesAppearance = 0;
 
 int currentTilesIdx = 0;
+int frame = 0;
+GLfloat fps;
+
 
 typedef enum {
-    OSM = 0, CLOUDMADE = 1, GOOGLE = 2, BING = 3, YAHOO = 4
+    LANDSCAPE = 0, PORTRAIT = 1
+}
+    Orientation;
+
+struct {
+    int mipmapToggle;
+    int showGrid, showCoordinates, accelerometerEnabled;
+    int showDebugTiles;
+    Orientation orientation;
+    char *userAgent;
+    char *referer;
+} options;
+
+typedef struct {
+    char* name;
+    int minZoom, maxZoom;
+    char* filenameFormat;
+    char* urlFormat;
 } TileProvider;
 
 typedef enum {
@@ -94,13 +95,16 @@ typedef struct {
 } TileCoordinate;
 
 struct {
-    GLfloat x, y, dx, dy, scale, oldScale, destScale, rotx, rotz, destRotx, drotz;
+    GLfloat x, y, dx, dy, scale, oldScale, destScale, rotx, roty, rotz, destRotx, destRoty, destRotz, drotz;
     int tilex, tiley;
-    int zoom, oldZoom, maxZoom;
+    int zoom, oldZoom;
     WorldCoordinate center;
-    TileProvider provider;
+    TileProvider* provider;
     ViewMode viewMode;
-    int showGrid, showCoordinates, accelerometerEnabled;
+    GLfloat orientationTransition;
+    int orientationTransitionLinear;
+    int followingMypos;
+    TileCoordinate boundingTrapezoid[4]; // points in clockwise order
 } canvas;
 
 struct {
@@ -122,10 +126,6 @@ typedef struct {
     GLfloat r, g, b, a;
 } Color;
 
-typedef enum {
-    ANCHOR_LEFT, ANCHOR_UP, ANCHOR_CENTER, ANCHOR_DOWN, ANCHOR_RIGHT
-} Anchor;
-
 typedef enum {
     UI_HIDDEN = 0, UI_COMING = 1, UI_SHOWN = 2, UI_GOING = 3
 } uiElemStatus;
@@ -136,7 +136,10 @@ typedef struct {
 } Texture;
 
 typedef struct {
+    GLfloat portraitx, portraity;
+    GLfloat landscapex, landscapey;
     GLfloat x, y;
+    GLfloat destx, desty;
     long stateChangeMilis;
     uiElemStatus status;
     Texture texture;
@@ -144,33 +147,39 @@ typedef struct {
     Color color;
     void (*handlePressed)();
     void (*handleDragged)();
+    GLfloat * texCoords;
 } UiElement;
 
-#include "queue.c"
-queue* uiElems;
+GQueue * downloadQueue;
+
+#include "tile.c"
+#include "downloadQueue.c"
+GList* uiElems = NULL;
 
 UiElement* pressedUiElem = NULL;
 
 UiElement *crosshair, *zoomIn, *zoomOut, *position, *gotomypos, *view2d, *view3d;
 
-#include "tile.c"
-t_tile* tiles[TILES_X][TILES_Y][2];
+GLfloat modelMatrix[16];
+GLfloat projectionMatrix[16];
+GLint viewPort[4];
 
 #include "network.c"
 #include "texture.c"
 #include "tileProviders.c"
+#include "glunproject.c"
 
 LocationGPSDevice *device;
 LocationGPSDControl *control;
 
 #include "tileengine.c"
 #include "glcanvas.c"
+#include "accelerometer.c"
 #include "input.c"
 #include "config.c"
 #include "location.c"
 #include "battery.c"
 #include "taskman.c"
-#include "accelerometer.c"
 
 int main(int argc, char **argv) {
     SDL_Surface *surface;
@@ -200,10 +209,6 @@ int main(int argc, char **argv) {
     initLocation();
     initBattery();
     curl_global_init(CURL_GLOBAL_ALL);
-    // TEMPORARY START
-    //    runTests();
-    //    return 0;
-    // TEMPORARY END
 
     /* wait for events */
     while (!quit) {
@@ -225,9 +230,6 @@ int main(int argc, char **argv) {
                 case SDL_QUIT:
                     quit = TRUE;
                     break;
-                case SDL_USEREVENT:
-                    handleGenTextureEvent(&event);
-                    break;
                 case SDL_KEYDOWN:
                     processKeyDown(event.key);
                     break;
@@ -242,26 +244,35 @@ int main(int argc, char **argv) {
         processKeyboard();
         processAccelerometer();
         tileEngineUpdateCoordinates();
+        setupGlScene();
+        tileEngine_computeBoundingTrapezoid();
+        tileEngine_computeTilesVisibility();
         updateUi();
         updateSyncWaitTiles();
+
+        // TODO detect screen lock
         if (!isActive) {
             // when our window is inactive save system resources by limiting framerate to 10 fps
             SDL_Delay(100);
         }
         drawGLScene();
 
+        // refresh battery percentage every 10 seconds
         if (nowMillis - lastBatteryRefresh > 10000) {
             refreshBattery();
             lastBatteryRefresh = nowMillis;
         }
+        processDeferredDeallocation();
+        frame++;
+
     }
 
     /* clean ourselves up and exit */
     // wait for all created threads here
     saveConfig();
+    SDL_GLES_DeleteContext(context);
     shutdownTileEngine();
     shutdownLocation();
-    SDL_GLES_DeleteContext(context);
     SDL_Quit();
     return 0;
 }
